/*!
* \file GroundEffect.cpp
* \brief Calculation of ground effect attenuations
* \author CSTB
* \version 1.0
*/

#include "GroundEffect.h"
#include <math.h>
#include <stdio.h>

namespace GroundEffectNMPB
{	
	// ------------------------------ MeanPlane Class -----------------------------------------

	/*!
	 * \brief Initialization of the MeanPlane class with given arguments, and data calculation
	 * 
	 * \param terrainItems The terrain item vector
	 */
	MeanPlane::MeanPlane(vector<ProfilePointNMPB*> terrainItems)
	{
		if (terrainItems.size() < 2)
		{
			return;
		}

		_source = terrainItems[0];
		_receiver = terrainItems[terrainItems.size() - 1];
		_terrainItems = terrainItems;

		// Fills the points list with the terrain items positions
		for (int i = 0 ; i < (int) terrainItems.size() ; i++)
		{
			ProfilePointNMPB* terrainItem = terrainItems[i];
			_pointsList.push_back(terrainItem->position2D);
		}

		CalculateData();

	}

	/*!
	* \brief Calculates the mean plane coefficients (a and b : z = ad + b) 
	* 
	* p. 69 - Annexe E
	* 
	*/
	void MeanPlane::FillLineCoefficients()
	{
		int pointsNumber = (int) _pointsList.size();
		if (_pointsList[0].d != _pointsList[pointsNumber-1].d)
		{
			// mean plane coefficients
			double valA1 = 0;
			double valA2 = 0;
			double valB1 = 0;
			double valB2 = 0;
			for (int i = 0 ; i < pointsNumber - 1 ; i++)
			{
				Position2D point_i = _pointsList[i];
				Position2D point_i1 = _pointsList[i+1];
				double diffdi = point_i1.d - point_i.d;
				if (diffdi != 0)
				{
					// zi = ai * di + bi
					double ai = (point_i1.z - point_i.z) / diffdi;
					double bi = point_i.z - ai * point_i.d;
					double vald2 = (pow(point_i1.d, 2) - pow(point_i.d, 2));
					valA1 += ai * (pow(point_i1.d, 3) - pow(point_i.d, 3));
					valA2 += bi * vald2;
					valB1 += ai * vald2;
					valB2 += bi * diffdi;
				}			
			}

			double valA = 2 * valA1 / 3 + valA2;
			double valB = valB1 + 2 * valB2;

			double dividende = pow(_pointsList[pointsNumber-1].d - _pointsList[0].d, 3);
			_aCoeff =  3 * (2 * valA - valB * (_pointsList[pointsNumber-1].d + _pointsList[0].d)) / dividende;
			_bCoeff = 2 * valB * (pow(_pointsList[pointsNumber-1].d, 3) - pow(_pointsList[0].d, 3))/ ((_pointsList[pointsNumber-1].d - _pointsList[0].d) * dividende) - 3 * (_pointsList[pointsNumber-1].d + _pointsList[0].d) * valA / dividende; 
		}	
	}

	/*!
	 * \brief Calculates the mean plane projection of a point
	 * 
	 * \param point The 2D point to be projected
	 * 
	 * \returns The projected 2D point
	 */
	Position2D MeanPlane::MeanPlaneProjection(Position2D const* point)
	{
		Position2D eqPoint;
		double dividende = 1 + _aCoeff * _aCoeff;
		eqPoint.d = (point->z * _aCoeff + point->d - _aCoeff * _bCoeff) / dividende;
		eqPoint.z = (point->z * _aCoeff * _aCoeff + point->d * _aCoeff + _bCoeff) / dividende;

		return eqPoint;
	}

	/*!
	* \brief Calculates distance between source and receiver on the mean plane, S image, R image, S equivalent height, R equivalent height and the ground coefficient
	* 
	* p. 38 -  9.3.1 ; p. 39 -  9.3.2 
	* 
	*/
	void MeanPlane::FillData()
	{
		Position2D _sourceWithHeight;
		_sourceWithHeight.d = _source->position2D.d;
		_sourceWithHeight.z = _source->position2D.z + _source->height;
		Position2D _receiverWithHeight;
		_receiverWithHeight.d = _receiver->position2D.d;
		_receiverWithHeight.z = _receiver->position2D.z + _receiver->height;

		Position2D eq1 = MeanPlaneProjection(&_sourceWithHeight);
		Position2D eq2 = MeanPlaneProjection(&_receiverWithHeight);

		if (eq1.z <= _sourceWithHeight.z)
		{
			_zEqS = distance2D(&eq1, &_sourceWithHeight);
			_imageS.d = 2 * eq1.d - _sourceWithHeight.d;
			_imageS.z = 2 * eq1.z - _sourceWithHeight.z;
		}
		else
		{
			_zEqS = 0;
			_imageS = eq1;
		}

		if (eq2.z <= _receiverWithHeight.z)
		{
			_zEqR = distance2D(&eq2, &_receiverWithHeight);
			_imageR.d = 2 * eq2.d - _receiverWithHeight.d;
			_imageR.z = 2 * eq2.z - _receiverWithHeight.z;
		}
		else
		{
			_zEqR = 0;
			_imageR = eq2;
		}

		_dp = distance2D(&eq1, &eq2);

		// ground coefficient calculation : 
		_Gpath = 0;
		int terrainsNb = (int) _terrainItems.size();
		if (terrainsNb != 0 )
		{		
			double sumG = 0;
			double sumD = 0;
			for (int i = 0; i < terrainsNb ; i++ )
			{
				ProfilePointNMPB* terrainItem = _terrainItems[i];
				if(i == 0)
				{
					sumD += terrainItem->position2D.d - _source->position2D.d;
					sumG += terrainItem->impedance * (terrainItem->position2D.d - _source->position2D.d);
				}
				else
				{
					sumD += terrainItem->position2D.d - _terrainItems[i-1]->position2D.d;
					sumG += terrainItem->impedance * (terrainItem->position2D.d - _terrainItems[i-1]->position2D.d);
				}
			}

			sumD += _receiver->position2D.d - _terrainItems[terrainsNb-1]->position2D.d;
			sumG += _receiver->impedance * (_receiver->position2D.d - _terrainItems[terrainsNb-1]->position2D.d);
			
			if(sumD != 0)
			{
				_Gpath = sumG / sumD;
			}		
		}
	}

	// ------------------------------ GroundEffect Class -----------------------------------------

	/*!
	 * \brief Calculates corrected ground sound absorption
	 * 
	 * Calculates Gpath' : correct Gpath depending on distances
	 * p. 39 -  9.3.2 formula (22)
	 * 
	 */
	void GroundEffect::CorrectedGroundCoeffCalculation()
	{
		double maxDp = 30 * (_zEqS + _zEqR);
		if (_dp <= maxDp && maxDp != 0)
		{
			_correctedGpath = _dp * _Gpath / maxDp + (1 - (_dp / maxDp)) * _Gsource ;
		}
		else
		{
			_correctedGpath = _Gpath;
		}
	}
    
	/*!
	 * \brief Calculates the w parameter
	 * 
	 * w parameter calculation : used in Cf calculation for ground attenuation
	 * p. 41 -  9.3.3 formula (25)
	 * 
	 * \param Gw : Gpath or corrected Gpath
	 * 
	 * \throws ERRDivZero
	 */
	double GroundEffect::WparamCalculation(double Gw)
	{
		double dividende = pow(_freq,1.5) * pow(Gw,2.6) + 1.3 * 1000 * pow(_freq,0.75) * pow(Gw,1.3) + 1.16 * 1000000;
		double wParam;
		if (dividende != 0)
		{
			wParam = 0.0185 * (pow(_freq,2.5) * pow(Gw,2.6))/ dividende;
		}
		else
		{
			printf ("Erreur : division par 0 dans le calcul de l'effet de sol (1)\n");
			throw new int(ERRDivZero);
		}

		return wParam;
	}

	/*!
	 * \brief Calculates the Cf parameter
	 * 
	 * Cf parameter calculation : used in the ground attenuation calculation
	 * p. 41 -  9.3.3 formula (24)
	 * 
	 * \throws ERRDivZero 
	 */
	void GroundEffect::CfCalculation()
	{
		double wdpH, wdpF;
		switch(_groundCalculationType)
		{
		case Asol:
			wdpH = WparamCalculation(_correctedGpath) * _dp;
			wdpF = WparamCalculation(_Gpath) * _dp;
			break;
		case DeltaSol_SO:
			wdpH = WparamCalculation(_correctedGpath) * _dp;
			wdpF = WparamCalculation(_Gpath) * _dp;
			break;
		case DeltaSol_OR:
			wdpH = WparamCalculation(_Gpath) * _dp;
			wdpF = wdpH;
			break;
		default:
			return;
		}
 
		if ((1 + wdpH) != 0)
		{
			_cfH = _dp * (1 + 3 * wdpH * exp(-1 * sqrt(wdpH))) / (1 + wdpH);
		}
		else
		{
			printf("Erreur : division par 0 dans le calcul de l'effet de sol (2)\n");
			throw new int(ERRDivZero);
		}

		if ((1 + wdpF) != 0)
		{
			_cfF = _dp * (1 + 3 * wdpF * exp(-1 * sqrt(wdpF))) / (1 + wdpF);
		}
		else
		{
			printf("Erreur : division par 0 dans le calcul de l'effet de sol (3)\n");
			throw new int(ERRDivZero);
		}
	}

	/*!
	 * \brief Calculates the k parameter
	 * 
	 * k parameter calculation : used in the ground attenuation calculation
	 * p. 41 -  9.3.3 formula (23)
	 * 
	 */
	void GroundEffect::KfreqCalculation()
	{
		_kFreq = 2 * PI * _freq / cSound;
	}

	/*!
	 * \brief : Calculates attenuation
	 * 
	 * \param z1 : source equivalent height
	 * 
	 * \param z2 : receiver equivalent height
	 * 
	 * \param cf : Cf parameter
	 * 
	 * \returns : calculated attenuation
	 * 
	 * Attenuation calculation depending on distances, heights, frequencies and ground type
	 * p. 41 -  9.3.3 formula (23) before calling max
	 * 
	 * \throws ERRDivZero, ERRSqrtNegative
	 */
	double GroundEffect::AttenuationCalculation(double z1, double z2, double cf)
	{
		double attSol = 0;
		if (_kFreq != 0 && _dp != 0)
		{
			double CfOnK = cf / _kFreq;
			if (CfOnK >= 0)
			{
				double sqrt2CfOnK = sqrt(2 * CfOnK);
				double mult1 = 4 * _kFreq * _kFreq / (_dp * _dp);
				double mult2 = (z1 * z1 - sqrt2CfOnK * z1 + CfOnK);
				double mult3 = (z2 * z2 - sqrt2CfOnK * z2 + CfOnK);

				attSol = -10 * log10(mult1 * mult2 * mult3);
			}
			else
			{
				printf("Erreur : racine carre d'un nombre ngatif dans le calcul de l'effet de sol \n");
				throw new int(ERRSqrtNegative);
			}
		}
		else
		{
			printf("Erreur : division par 0 dans le calcul de l'effet de sol (4)\n");
			throw new int(ERRDivZero);
		}

		return attSol;
	}

	/*!
	 * \brief Calculates homogeneous attenuation
	 * 
	 * \returns Asol,H : homogeneous attenuation
	 * 
	 * Attenuation calculation in homogeneous propagation conditions, depending on distances, heights, frequency and ground type
	 * p. 41 -  9.3.3 formula (23)	 
	 */
	double GroundEffect::AttenuationCalculationH()
	{
		double attSolH = -3; // default value if Gpath = 0

		if (_Gpath != 0)
		{
			attSolH = AttenuationCalculation(_zEqS, _zEqR, _cfH);

			// Checks minimal value :
			double attSolHmin; // formula (26)
			switch(_groundCalculationType)
			{
			case Asol:
				attSolHmin = -3 * (1 - _correctedGpath);
				break;
			case DeltaSol_SO:
				attSolHmin = -3 * (1 - _correctedGpath);
				break;
			case DeltaSol_OR:
				attSolHmin = -3 * (1 - _Gpath);
				break;
			}

			if (attSolH < attSolHmin)
			{
				attSolH = attSolHmin;
			}
		}

		_AsolH = attSolH; 
		return attSolH;
	}

	/*!
	 * \brief Calculates favorable attenuation
	 * 
	 * \returns Asol,F : favorable attenuation
	 * 
	 * Attenuation calculation in favorable propagation conditions, depending on distances, heights, frequency and ground type
	 * p. 42 -  9.3.4 using formula (23) and (27)
	 * 
	 * \throws ERRDivZero
	 */
	double GroundEffect::AttenuationCalculationF()
	{
	 // modification dvm 20.04.2011
	 // in case Gpath=0, we have AF = AFmin as decided at the meeting LRS/CSTD in Paris

	 // modification dvm 09.09.2013, annule et replace la prcdente
	 // pour le calcul en conditions homognes, le cas G=0 n'est pas considr comme un cas particulier
	 // on utilise toujours les formules gnrales (23)  (26)

		double zEqSum = _zEqS + _zEqR;
		double attSolFmin = 0;
		switch(_groundCalculationType)
		{
		case Asol:
			attSolFmin = -3 * (1 - _correctedGpath);
			break;
		case DeltaSol_SO:
			attSolFmin = -3 * (1 - _correctedGpath);
			break;
		case DeltaSol_OR:
			attSolFmin = -3 * (1 - _Gpath);
			break;
		}

		double maxDp = 30 * zEqSum;
		if (_dp > maxDp && _dp != 0)
		{
			attSolFmin *= (1 + 2 *(1 - maxDp / _dp));
		}

		// default value if Gpath = 0, interprtation du groupe d'experts NMPB-2008
		// dcision confirm par Guillaume Dutillieux le ??

		double attSolF = -3 ;

		if (_Gpath != 0)
		{
			// modification dvm 18.05.2013
			// traiter le cas zEqSum = 0 !!

			if ((zEqSum) != 0)
			{
				double a0 = 0.0002;
				double dp2by2 = _dp * _dp / 2; 
				double delta_zS = a0 * pow((_zEqS/(zEqSum)), 2) * dp2by2;
				double delta_zR = a0 * pow((_zEqR/(zEqSum)), 2) * dp2by2;
				double delta_zT = 0.006 * _dp / (zEqSum);
				_zEqS_f = _zEqS + delta_zS + delta_zT;
				_zEqR_f = _zEqR + delta_zR + delta_zT;
			}
			else
			{
				_zEqS_f = _zEqS ;
				_zEqR_f = _zEqR ;
			}
			
			attSolF = AttenuationCalculation(_zEqS_f, _zEqR_f, _cfF);

			// Checks minimal value :

			if (attSolF < attSolFmin)
			{
				attSolF = attSolFmin;
			}
		}

		_AsolF = attSolF;
		return attSolF;
	}
}