/*!
* \file GroundEffect.h
* \brief Calculation of ground effect attenuations
* \author CSTB
* \version 1.0
*/

#ifndef GROUND_EFFECT_H
#define GROUND_EFFECT_H

#include "../PathStructures.h"

#ifndef _WIN32
#define  MeanPlane		_Local_PROPAN8_MeanPlane_
#define  GroundEffect	_Local_PROPAN8_GroundEffect_
#endif


namespace GroundEffectNMPB
{
	/*!
	 * \brief Ground Calculation Type used to know when using _Gpath or _correctedGpath in the ground attenuation calculation
	 */
	typedef enum 
	{ 
		Asol = 1 , 
		DeltaSol_SO = 2 ,  
		DeltaSol_OR = 3
	} GroundCalculationType;

	/*!
	* \brief Class used to calculate the mean plane.
	* 
	* p. 69 - Annexe E
	* 
	*/
	class MeanPlane
	{
	public:
		/*!
		* \brief Initialization of the MeanPlane class
		*/
		MeanPlane()
		{
			_dp = 0;
			_zEqS = 0;
			_zEqR = 0;
			_Gpath = 0;
			_aCoeff = 0;
			_bCoeff = 0;
		}

		/*!
		* \brief Initialization of the MeanPlane class with given arguments, and data calculation
		* 
		* \param terrainItems The terrain item vector
		*/
		MeanPlane(vector<ProfilePointNMPB*> terrainItems);

		/*!
		 * \brief Gets distance between source and receiver on the mean plane
		 * 
		 * \returns the distance between source and receiver on the mean plane
		 */
		double Get_dp()
		{
			return _dp;
		}
		
		/*!
		* \brief Gets Source equivalent height
		* 
		* \returns the Source equivalent height
		*/
		double Get_zEqSource()
		{
			return _zEqS;
		}

		/*!
		* \brief Gets Receiver equivalent height
		* 
		* \returns the Receiver equivalent height
		*/
		double Get_zEqReceiver()
		{
			return _zEqR;
		}

		/*!
		* \brief Gets ground coefficient
		* 
		* \returns the ground coefficient
		*/
		double Get_Gpath()
		{
			return _Gpath;
		}

		/*!
		* \brief Gets the source image by the mean plane
		* 
		* \returns  the source image by the mean plane
		*/
		Position2D Get_sourceImage()
		{
			return _imageS;
		}
		
		/*!
		* \brief Gets the receiver image by the mean plane
		* 
		* \returns  the receiver image by the mean plane
		*/
		Position2D Get_receiverImage()
		{
			return _imageR;
		}

		/*!
		* \brief Gets the "a" coefficient for the mean plane line equation z = ad + b
		* 
		* \returns the "a" coefficient for the mean plane line equation z = ad + b
		*/
		double Get_aCoeff()
		{
			return _aCoeff;
		}

		/*!
		* \brief Gets the "b" coefficient for the mean plane line equation z = ad + b
		* 
		* \returns the "b" coefficient for the mean plane line equation z = ad + b
		*/
		double Get_bCoeff()
		{
			return _bCoeff;
		}

		/*!
		 * \brief Gets the impedance source
		 * 
		 * \returns the impedance source
		 */
		double Get_Gsource()
		{
			if (_source != NULL)
			{
				return _source->impedance;
			}

			return 0;
		}

	protected:
		/*!
		 * \brief Source
		 */
		ProfilePointNMPB* _source; 
		/*!
		* \brief Receiver
		*/
		ProfilePointNMPB* _receiver; 
		/*!
		* \brief terrain items
		*/
		vector<ProfilePointNMPB*> _terrainItems;
		/*!
		* \brief items coordinates list
		*/
		vector<Position2D> _pointsList;
		/*!
		 * \brief the mean plane a coefficient (a and b : z = ad + b)
		 */
		double _aCoeff; 
		/*!
		* \brief the mean plane b coefficient (a and b : z = ad + b)
		*/
		double _bCoeff;
		/*!
		* \brief distance between source and receiver on the mean plane
		*/
		double _dp;
		/*!
		* \brief S equivalent height
		*/
		double _zEqS;
		/*!
		* \brief R equivalent height
		*/
		double _zEqR;
		/*!
		* \brief ground coefficient
		*/
		double _Gpath;
		/*!
		* \brief source image by the mean plane
		*/
		Position2D _imageS;
		/*!
		* \brief receiver image by the mean plane
		*/
		Position2D _imageR;

		/*!
		* \brief Calculates the mean plane coefficients (a and b : z = ad + b) 
		* 
		* p. 69 - Annexe E
		* 
		*/
		void FillLineCoefficients();

		/*!
		* \brief Calculates the mean plane projection of a point
		* 
		* \param point The 2D point to be projected
		* 
		* \returns The projected 2D point
		*/
		Position2D MeanPlaneProjection(Position2D const* point);

		/*!
		* \brief Calculates distance between source and receiver on the mean plane, S image, R image, S equivalent height, R equivalent height and the ground coefficient
		* 
		* p. 38 -  9.3.1 ; p. 39 -  9.3.2 
		* 
		*/
		void FillData();

		/*!
		* \brief Calculates the mean plane coefficients and then the equivalent heights and the distance on the mean plane
		*/
		void CalculateData()
		{
			FillLineCoefficients();
			FillData();
		}
	};


    /*!
	 * \brief Class used to calculate ground effects.
	 * 
	 * p. 38-44 -  9.3
	 * 
	 */
	class GroundEffect
    {
    public:
		
		/*!
		* \brief Initialization of the GroundEffect class
		*/
		GroundEffect()
		{
			_dp = 0;
			_zEqS = 0;
			_zEqR = 0;
			_zEqS_f = 0;
			_zEqR_f = 0;
			_freq = 0;
			_Gpath = 0;
			_Gsource = 0;
			_correctedGpath = 0;
			_groundCalculationType = Asol;
			_kFreq = 0;
			_cfH = 0; 
			_cfF = 0;
			 _AsolH = 0;
			_AsolF = 0;
		}

		/*!
		 * \brief Initialization of the GroundEffect class
		 * 
		 * \param dp : distance between source and receiver on the mean plane
		 * 
		 * \param zEqS : S equivalent height
		 * 
		 * \param zEqR : R equivalent height
		 *
		 * \param Gpath : ground coefficient
		 *
		 * \param Gsource : source ground coefficient
		 * 
		 * \param freq : center frequency
		 * 
		 * \param groundCalculationType : ground calculation type (Asol, DeltaSol_SO or DeltaSol_OR)
		 * 
		 * GroundEffect Initialization with SR mean plane distance, equivalent heights, and center frequency
		 * 
		 */
		GroundEffect(double dp, double zEqS, double zEqR, double Gpath, double Gsource, double freq, GroundCalculationType groundCalculationType)
		{
			_dp = dp;
			_zEqS = zEqS;
			_zEqR = zEqR;
			_zEqS_f = 0;
			_zEqR_f = 0;
			_freq = freq;
			_Gpath = Gpath;
			_Gsource = Gsource;
			_correctedGpath = 0;
			_groundCalculationType = groundCalculationType;
			_kFreq = 0;
			_cfH = 0; 
			_cfF = 0;
			_AsolH = 0;
			_AsolF = 0;

			// Calculates values used to get attenuation :
			CorrectedGroundCoeffCalculation();
			CfCalculation();
			KfreqCalculation();
		}

		/*!
		* \brief Calculates homogeneous attenuation
		* 
		* \returns Asol,H : homogeneous attenuation
		* 
		* Attenuation calculation in homogeneous propagation conditions, depending on distances, heights, frequency and ground type
		* p. 41 -  9.3.3 formula (23)	 
		*/
        double AttenuationCalculationH();

		/*!
		* \brief Calculates favorable attenuation
		* 
		* \returns Asol,H : favorable attenuation
		* 
		* Attenuation calculation in favorable propagation conditions, depending on distances, heights, frequency and ground type
		* p. 42 -  9.3.4 using formula (23) and (27)
		* 
		* \throws ERRDivZero
		*/
		double AttenuationCalculationF();

		/*!
		 * \brief Gets ground effect attenuation in homogeneous conditions
		 * 
		 * \returns ground effect attenuation in homogeneous conditions
		 */
		double get_AsolH()
		{
			return _AsolH;
		}
		/*!
		* \brief Gets ground effect attenuation in favorable conditions
		* 
		* \returns ground effect attenuation in favorable conditions
		*/
		double get_AsolF()
		{
			return _AsolF;
		}

		/*!
		* \brief Gets equivalent height of S measured perpendicular to the mean ground plane
		* 
		* \returns equivalent height of S measured perpendicular to the mean ground plane
		*/
		double Get_zEqSource()
		{
			return _zEqS;
		}

		/*!
		* \brief Gets equivalent height of R measured perpendicular to the mean ground plane
		* 
		* \returns equivalent height of R measured perpendicular to the mean ground plane
		*/
		double Get_zEqReceiver()
		{
			return _zEqR;
		}

		/*!
		* \brief Gets equivalent height of S measured perpendicular to the mean ground plane in favorable conditions
		* 
		* \returns equivalent height of S measured perpendicular to the mean ground plane in favorable conditions
		*/
		double Get_zEqSource_f()
		{
			return _zEqS_f;
		}

		/*!
		* \brief Gets equivalent height of R measured perpendicular to the mean ground plane in favorable conditions
		* 
		* \returns equivalent height of R measured perpendicular to the mean ground plane in favorable conditions
		*/
		double Get_zEqReceiver_f()
		{
			return _zEqR_f;
		}

		/*!
		* \brief Gets equivalent ground coefficient along a propagation path
		* 
		* \returns equivalent ground coefficient along a propagation path
		*/
		double get_EquivalentGpath()
		{
			return _correctedGpath;
		}

	protected:
		/*!
		 * \brief distance between source and receiver on the mean plane 
		 */
		double _dp;
		/*!
		* \brief S equivalent height in homogeneous conditions 
		*/
		double _zEqS;
		/*!
		* \brief R equivalent height in homogeneous conditions
		*/
		double _zEqR;
		/*!
		* \brief S equivalent height in favorable conditions
		*/
		double _zEqS_f;
		/*!
		* \brief R equivalent height in favorable conditions 
		*/
		double _zEqR_f;
		/*!
		* \brief center frequency fm
		*/
		double _freq;
		/*!
		* \brief ground coefficient 
		*/
		double _Gpath;
		/*!
		* \brief source ground coefficient 
		*/
		double _Gsource;
		/*!
		* \brief corrected ground coefficient (with d and z) 
		*/
		double _correctedGpath;
		/*!
		* \brief ground Calculation Type (Enum) used to know when using _Gpath or _correctedGpath
		*/
		GroundCalculationType _groundCalculationType;
		
		/*!
		* \brief k parameter depending on the frequency fm
		*/
		double _kFreq;
		/*!
		* \brief Cf parameter depending on the distance and w, for homogeneous or favorable conditions 
		*/
		double _cfH, _cfF;
 
		/*!
		* \brief ground effect attenuation in homogeneous conditions
		*/
		double _AsolH;
		/*!
		* \brief ground effect attenuation in favorable conditions 
		*/
		double _AsolF;


		/*!
		* \brief Calculates corrected ground sound absorption
		* 
		* Calculates Gpath' : correct Gpath depending on distances
		* p. 39 -  9.3.2 formula (22)
		* 
		*/
		void CorrectedGroundCoeffCalculation();
		/*!
		* \brief Calculates the w parameter
		* 
		* w parameter calculation : used in Cf calculation for ground attenuation
		* p. 41 -  9.3.3 formula (25)
		* 
		* \param Gw : Gpath or corrected Gpath
		* 
		* \throws ERRDivZero
		*/
		double WparamCalculation(double Gw);
		/*!
		* \brief Calculates the Cf parameter
		* 
		* Cf parameter calculation : used in the ground attenuation calculation
		* p. 41 -  9.3.3 formula (24)
		* 
		* \throws ERRDivZero 
		*/
		void CfCalculation();
		/*!
		* \brief Calculates the k parameter
		* 
		* k parameter calculation : used in the ground attenuation calculation
		* p. 41 -  9.3.3 formula (23)
		* 
		*/
		void KfreqCalculation();

		/*!
		* \brief : Calculates attenuation
		* 
		* \param z1 : source equivalent height
		* 
		* \param z2 : receiver equivalent height
		* 
		* \param cf : Cf parameter
		* 
		* \returns : calculated attenuation
		* 
		* Attenuation calculation depending on distances, heights, frequencies and ground type
		* p. 41 -  9.3.3 formula (23) before calling max
		* 
		* \throws ERRDivZero, ERRSqrtNegative
		*/
		double AttenuationCalculation(double z1, double z2, double cf);

	};
}

#endif