/*!
* \file pathdefNMPB.h
* \brief Definition of main enumerations and structures used in the call of the library functions
* \author CSTB
* \version 1.0
*/

#ifndef _PATHDEF_NMPB_INCLUDED_
#define _PATHDEF_NMPB_INCLUDED_


/*!
 * \brief Error types send by functions
 */
enum ErrorType
{
	/*!
	 * \brief No error
	 */
	ERRNone = 0,
	/*!
	 * \brief The PathID path is null
	 */
	ERRNullPath = 10,
	/*!
	* \brief There is no point in the path
	*/
	ERRNoPoint = 11,
	/*!
	* \brief There is only one point in the path
	*/
	ERROnePoint = 12,
	/*!
	* \brief There are more than 2 side diffractions (ProfilePointNMPB with ext.type = ETSideDiffraction_NMPB) in the path
	*/
	ERRSideDiff = 13,
	/*!
	* \brief There are more than 1 embankment (ProfilePointNMPB with ext.type = ETEmbankment_NMPB) in the path
	*/
	ERREmbankment = 14,
	/*!
	* \brief The propagation distance exceeds the maximum value of 2000m
	*/
	ERRMaxDistance = 15,
	/*!
	* \brief The frequency equals 0
	*/
	ERRFrequency = 20,
	/*!
	* \brief Frequency not found to get attenuation coefficient
	*/
	ERRAttCoeffFrequency = 21,
	/*!
	* \brief Angle not found in the array containing favorable conditions probabilities for the angles
	*/
	ERRAngle = 30,
	/*!
	* \brief The probability is greater than 1
	*/
	ERRProbability = 40,
	/*!
	* \brief Division by zero
	*/
	ERRDivZero = 50,
	/*!
	* \brief Square root of negative number
	*/
	ERRSqrtNegative = 51,
	/*!
	* \brief The given screen absorption is greater than 1
	*/
	ERRScreenAbsorption = 60,
	/*!
	* \brief None expected error
	*/
	ERRUnknown = 100,
} ;

/*!
 * \brief Options that can be chosen by user
 */
enum Option
{
	/*!
	 * \brief Don't calculate Adiv
	 */
	EXCLUDE_ADIV		= 1 << 1, 
	/*!
	 * \brief Don't calculate Aatm
	 */
	EXCLUDE_AATM		= 1 << 2, 
	/*!
	* \brief Displays trace details on console
	*/
	TRACE_DETAILS		= 1 << 3, 
	/*!
	* \brief Check embankments on the path
	*/
	CHECK_EMBANKMENT	= 1 << 4, 
	/*!
	* \brief Disable correction term for diffraction by low height obstacles, always use Ch = 1
	*/
	FORCE_CH_EQUAL_ONE = 1 << 5,
};

/*!
* \brief extension type
*/
enum ExtensionTypeNMPB
{
	/*!
	 * \brief No particular extension
	 */
	ETNone_NMPB					= 0,
	/*!
	 * \brief Screen extension : height must be filled
	 */
	ETScreen_NMPB				= 1,
	/*!
	* \brief Reflection extension : height and alphaArray must be filled
	*/
	ETReflection_NMPB			= 2,
	/*!
	* \brief Side diffraction extension : height must be filled 
	*/
	ETSideDiffraction_NMPB		= 3,
	/*!
	* \brief Platform extension
	*/
	ETPlatform_NMPB				= 4,
	/*!
	* \brief Embankment extension : cosTheta must be filled
	*/
	ETEmbankment_NMPB			= 5,
	/*!
	* \brief Road source extension
	*/
	ETRoadSource_NMPB			= 6,
} ;

/*!
* \brief Extension for path elements
*/
struct ExtensionNMPB
{
	/*!
	* \brief the extension type
	*/
	ExtensionTypeNMPB	type ;
	/*!
	* \brief the height
	*/
	double				height ;
	/*!
	* \brief absorption coefficients of the surface, for each frequency
	*/
	double*				alphaArray ;
	/*!
	* \brief the Theta cosines where Theta is the angle between the perpendicular to the road and the propagation path 
	*/
	double				cosTheta ;
} ;

/*!
* \brief 3D point coordinates
*/
typedef struct
{
	double    x, y, z ;
} Position3D ;

/*!
* \brief 2D point coordinates in the vertical plane containing Source and Receptor
*/
typedef struct
{
	double    d, z ;
} Position2D ;

#endif
